/*-
 * Copyright (c) 2015 Weongyo Jeong <weongyo@3rocks.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

var g_conf_pac_config_full_mode = `function FindProxyForURL(url, host) {
  host = host.toLowerCase();
  if (isPlainHostName(host) ||
      isInNetEx(host, "127.0.0.0/8") ||
      isInNetEx(host, "10.0.0.0/8") ||
      isInNetEx(host, "172.16.0.0/12") ||
      isInNetEx(host, "192.168.0.0/16") ||
      isInNetEx(host, "fc00::/7") ||
      isInNetEx(host, "fe80::/10") ||
      host == "mudfish.net" ||
      host == "www.mudfish.net") {
        return "DIRECT";
  }
  return "{0}";
}`;

var g_conf = {
  check_current_ip: true,
  refresh_active_tab: false,
  use_secure_connection: false,
  use_ssl_bump: false,
  remember_me: false,
  username: "",
  password: "",
  pac_config: "",
  pac_config_custom_mode: "",
  pac_config_item_mode: "",
  pac_storage: {},                            // used for custom mode
  nodelist_policy: 0,
  nodelist_picked_nodes: [],
  traffic_mode: "full",
  item_mode_domains: [],
  item_mode_selected_items: []
}

var g_mudfish_items = [];

function mud_log(str) {
  console.log(new Date() + " " + str);
}

function mudfishItemFetch() {
  fetch('https://mudfish.net/api/webext/items')
    .then(response => {
      if (!response.ok) {
        throw new Error('Network response was not ok');
      }
      return response.json();
    })
    .then(data => {
      g_mudfish_items = data.items;
      mudfishItemRender();
    })
    .catch(error => {
      mud_log(`Error fetching Mudfish items: ${error}`);
      m.render(document.getElementById("item_mode_services"), 
        m("div.text-center.py-4.text-red-500", [
          m("i.fa.fa-exclamation-circle.mr-2"),
          chrome.i18n.getMessage("i18n_b2d130df38e3ff462d301a3e04878f9a")
        ])
      );
    });
}

function mudfishItemRender() {
  if (g_mudfish_items.length === 0) {
    m.render(document.getElementById("item_mode_services"), 
      m("div.text-center.py-4.text-gray-500", [
        m("i.fa.fa-info-circle.mr-2"),
        chrome.i18n.getMessage("i18n_2d4f13b1eb05227baea17c39edb26910")
      ])
    );
    return;
  }
  
  let items = [];
  g_mudfish_items.forEach(item => {
    const isChecked = g_conf.item_mode_selected_items && g_conf.item_mode_selected_items.includes(item.we_iid);
    
    items.push(
      m("div.flex.items-center.p-2.hover:bg-gray-50.border-b.border-gray-100", [
        m("label.flex.items-center.cursor-pointer.w-full", [
          m("input[type=checkbox].mudfish-item-checkbox.mr-3.w-4.h-4.accent-yellow-500", {
            "data-item-id": item.we_iid,
            checked: isChecked,
            onchange: function() {
              mudfishItemUpdateSelected();
            }
          }),
          m("img", {
            src: item.img_uri,
            alt: item.name,
            class: "w-4 h-4 mr-2"
          }),
          m("span.font-medium", item.name)
        ])
      ])
    );
  });
  
  m.render(document.getElementById("item_mode_services"), items);
}

function mudfishItemUpdateSelected() {
  if (!g_conf.item_mode_selected_items) {
    g_conf.item_mode_selected_items = [];
  }
  
  g_conf.item_mode_selected_items = [];
  
  $(".mudfish-item-checkbox:checked").each(function() {
    const itemId = parseInt($(this).data("item-id"));
    g_conf.item_mode_selected_items.push(itemId);
  });
  
  mudfishItemUpdateDomains();
  
  saveOptions();
}

function mudfishItemUpdateDomains() {
  let domains = [];
  
  g_conf.item_mode_selected_items.forEach(itemId => {
    const item = g_mudfish_items.find(i => i.we_iid === itemId);
    if (item && item.hostnames) {
      domains = domains.concat(item.hostnames);
    }
  });
  
  domains = [...new Set(domains)];
  
  g_conf.item_mode_domains = domains;
}

function sendAuth() {
  chrome.runtime.sendMessage({
    type: 'SET_AUTH',
    username: g_conf.username,
    password: g_conf.password,
  }, function (response) {
    if (response.error_code != 0) {
      mud_log("Failed to send 'SET_AUTH' command.");
    }
  });
}

function sendPacConfig() {
  generatePacConfig();
  chrome.runtime.sendMessage({
    type: 'SET_PAC_CONFIG',
    pac_config: g_conf.pac_config
  }, function (response) {
    if (response.error_code != 0) {
      mud_log("Failed to send 'SET_PAC_CONFIG' command.");
    } else {
      chrome.runtime.sendMessage({
        type: 'GET_STATUS'
      }, function (r1) {
        if (r1.error_code != 0) {
          mud_log("[ERROR] Failed to send 'GET_STATUS' command.");
          return;
        }
        if (!r1.connected) {
          return;
        }
        $("#options_menu_apply_warning").css("display", "");
      });
    }
  });
}

function sendPacConfigAndReconnect() {
  generatePacConfig();
  chrome.runtime.sendMessage({
    type: 'SET_PAC_CONFIG',
    pac_config: g_conf.pac_config
  }, function (response) {
    if (response.error_code != 0) {
      mud_log("Failed to send 'SET_PAC_CONFIG' command.");
    } else {
      applyPacConfig();
    }
  });
}

function applyPacConfig() {
  chrome.runtime.sendMessage({
    type: 'GET_STATUS'
  }, function (r1) {
    if (r1.error_code != 0) {
      mud_log("[ERROR] Failed to send 'GET_STATUS' command.");
      return;
    }
    if (!r1.connected) {
      $.notify(chrome.i18n.getMessage("i18n_7b277018e43d41bc445731092b91547d"), {
        autoHideDelay: 1000,
        className: "warn",
        position: "bottom left"
      });
      return;
    }
    chrome.runtime.sendMessage({
      type: 'DISCONNECT'
    }, function (r2) {
      if (r2.error_code != 0) {
        mud_log("[ERROR] Failed to send 'DISCONNECT' command.");
        return;
      }
      chrome.runtime.sendMessage({
        type: 'CONNECT',
        sid: r1.sid,
        use_secure_connection: r1.use_secure_connection,
        use_ssl_bump: r1.use_ssl_bump
      }, function (r3) {
        if (r2.error_code != 0) {
          mud_log("[ERROR] Failed to send 'CONNECT' command.");
          return;
        }
        $.notify(chrome.i18n.getMessage("i18n_733d43480c8589b1368e5def6b480415"), {
          className: "success",
          position: "bottom left"
        });
        $("#options_menu_apply_warning").css("display", "none");
      });
    });
  });
}

var g_proxy_domain_dump_enabled = false;
var g_domain_dump_timer = null;

function restoreDumpDomainOption() {
  chrome.runtime.sendMessage({
    type: 'GET_STATUS'
  }, function (resp) {
    g_proxy_domain_dump_enabled = resp.domain_dump_enabled;
    if (g_proxy_domain_dump_enabled) {
      $("#options_advanced_domain_dump_play_fa").removeClass("fa-play").addClass("fa-stop");
      $("#options_advanced_domain_dump_play_status").html(chrome.i18n.getMessage("i18n_c5564d2e8b8e0ae08bf4363f2b947166"));
      enableDumpDomain();
    } else {
      $("#options_advanced_domain_dump_play_fa").removeClass("fa-stop").addClass("fa-play");
      $("#options_advanced_domain_dump_play_status").html(chrome.i18n.getMessage("i18n_c23e2b09ebe6bf4cb5e2a9abe85c0be2"));
      disableDumpDomain();
    }
  });
}

function getDumpDomainHostnames() {
  chrome.runtime.sendMessage({
    type: 'GET_DUMP_DOMAIN'
  }, function (response) {
    response.hostnames.sort();
    $("#options_advanced_domain_dump_result").val(response.hostnames.join("\n"));
  });
}

function timerDumpDomain() {
  getDumpDomainHostnames();
  if (g_domain_dump_timer != null) {
    window.clearTimeout(g_domain_dump_timer);
  }
  g_domain_dump_timer = window.setTimeout(timerDumpDomain, 3000);
}

function enableDumpDomain() {
  chrome.runtime.sendMessage({
    type: 'SET_DUMP_DOMAIN',
    enable: true
  }, function (response) {
    g_proxy_domain_dump_enabled = true;
    if (g_domain_dump_timer != null) {
      window.clearTimeout(g_domain_dump_timer);
    }
    g_domain_dump_timer = window.setTimeout(timerDumpDomain, 3000);
    $("#options_advanced_domain_dump_play_fa").removeClass("fa-play").addClass("fa-stop");
    $("#options_advanced_domain_dump_play_status").html(chrome.i18n.getMessage("i18n_c5564d2e8b8e0ae08bf4363f2b947166"));
  });
}

function disableDumpDomain() {
  chrome.runtime.sendMessage({
    type: 'SET_DUMP_DOMAIN',
    enable: false
  }, function (response) {
    g_proxy_domain_dump_enabled = false;
    $("#options_advanced_domain_dump_play_fa").removeClass("fa-stop").addClass("fa-play");
    $("#options_advanced_domain_dump_play_status").html(chrome.i18n.getMessage("i18n_c23e2b09ebe6bf4cb5e2a9abe85c0be2"));
  });
}

function update_nodelist_select() {
  $("#nodelist_select").select2({
    placeholder: chrome.i18n.getMessage("i18n_d22774b05386897eb6d79672a3a41310"),
    allowClear: true,
    width: '100%'
  }).on("change", function(e) {
    g_conf.nodelist_picked_nodes = [];
    $("#nodelist_select option").each(function() {
      if ($(this).prop('selected')) {
        g_conf.nodelist_picked_nodes.push(parseInt($(this).val()));
      }
    });
    saveOptions();
  });
}

function handle_nodelist_options() {
  chrome.runtime.sendMessage({
    type: 'GET_STATICNODES_ALL'
  }, function (response) {
    var staticnodes = response.staticnodes;
    var countrycodes = {};
    staticnodes.forEach(staticnode => {
      var location = staticnode.location;

      countrycodes[location.substring(0, 2)] = true;
    });
    var links = ``;
    for (const [key, value] of Object.entries(countrycodes)) {
      links += ` <a href="#" id="nodelist_cc_${key}">${key}</a>`;
    }
    $("#nodelist_countrycodes").html(links); 
    for (const [key, value] of Object.entries(countrycodes)) {
      $(`#nodelist_cc_${key}`).click(function() {
        $("#nodelist_select option").each(function() {
          var cc = $(this).html().trim().substring(0, 2);
      
          if (cc == key) {
            $(this).prop('selected', true);
            g_conf.nodelist_picked_nodes.push(parseInt($(this).val()));
            saveOptions();
          }
        });
        update_nodelist_select();   
      });
    }
    var optgroups = ``;
    for (const [key, value] of Object.entries(countrycodes)) {
      optgroups += `<optgroup label="${key}" value="${key}">`;
      staticnodes.forEach(staticnode => {
        var location = staticnode.location;
        var countrycode = location.substring(0, 2);

        if (countrycode == key) {
          var selected = "";
          g_conf.nodelist_picked_nodes.forEach(sid => {
            if (staticnode.sid == sid) {
              selected = "selected='true'";
            }
          });
          optgroups += `<option value="${staticnode.sid}" ${selected} class="select2-result-selectable">`;
          optgroups += staticnode.location;
          optgroups += `</option>`;
        }
      });
      optgroups += `</optgroup>`;
    }
    $("#nodelist_select").append(optgroups);
    update_nodelist_select();
  });
  $("input[name='nodelist_policy']").click(function() {
    g_conf.nodelist_policy = parseInt($(this).val());
    saveOptions();
  });
  if (g_conf.nodelist_policy) {
    $("input[name='nodelist_policy'][value='1']").prop('checked', true);
  } else {
    $("input[name='nodelist_policy'][value='0']").prop('checked', true);
  }
}

function restoreOptions() {
  chrome.storage.local.get(g_conf, function (items) {
    g_conf = items;
    if (g_conf.check_current_ip) {
      $("#check_current_ip").prop("checked", true);
    } else {
      $("#check_current_ip").removeProp("checked");
    }
    if (g_conf.refresh_active_tab) {
      $("#refresh_active_tab").prop("checked", true);
    } else {
      $("#refresh_active_tab").removeProp("checked");
    }
    if (g_conf.use_secure_connection) {
      $("#use_secure_connection").prop("checked", true);
    } else {
      $("#use_secure_connection").removeProp("checked");
    }
    if (g_conf.use_ssl_bump) {
      $("#use_ssl_bump").prop("checked", true);
    } else {
      $("#use_ssl_bump").removeProp("checked");
    }
    if (g_conf.remember_me) {
      $("#remember_me").prop("checked", true);
      $("#username").removeProp("disabled");
      $("#username").val(g_conf.username);
      $("#password").removeProp("disabled");
      $("#password").val(g_conf.password);
      sendAuth();
    } else {
      $("#remember_me").removeProp("checked");
    }
    if (g_conf.pac_config_custom_mode === undefined ||
        g_conf.pac_config_custom_mode.length === 0) {
      if (g_conf.pac_config === undefined || g_conf.pac_config.length === 0) {
        g_conf.pac_config_custom_mode = g_conf_pac_config_full_mode;
      } else {
        g_conf.pac_config_custom_mode = g_conf.pac_config;
      }
    }
    $("#pac_config").val(g_conf.pac_config_custom_mode);
    handle_nodelist_options();
    init_pac_app();
    
    if (g_conf.traffic_mode === "item") {
      $("#mode_item").prop("checked", true);
      g_conf.pac_config = g_conf.pac_config_item_mode;
    } else if (g_conf.traffic_mode === "custom") {
      $("#mode_custom").prop("checked", true);
      g_conf.pac_config = g_conf.pac_config_custom_mode;
    } else {
      $("#mode_full").prop("checked", true);
      g_conf.pac_config = g_conf_pac_config_full_mode;
    }
    updateItemModeVisibility();
  });
  restoreDumpDomainOption();
  getDumpDomainHostnames();
}

function generatePacConfig() {
  if (g_conf.traffic_mode === "item") {
    let pac_template = `function FindProxyForURL(url, host) {
  host = host.toLowerCase();
  if (isPlainHostName(host) ||
      isInNetEx(host, "127.0.0.0/8") ||
      isInNetEx(host, "10.0.0.0/8") ||
      isInNetEx(host, "172.16.0.0/12") ||
      isInNetEx(host, "192.168.0.0/16") ||
      isInNetEx(host, "fc00::/7") ||
      isInNetEx(host, "fe80::/10") ||
      host == "mudfish.net" ||
      host == "www.mudfish.net") {
        return "DIRECT";
  }
`;
    if (g_conf.item_mode_domains && g_conf.item_mode_domains.length > 0) {
      pac_template += '  if (';
      const domainPatterns = g_conf.item_mode_domains.map(domain => {
        return `host == "${domain}"`;
      });
      pac_template += domainPatterns.join(' ||\n      ');
      pac_template += ') {\n';
      pac_template += '    return "{0}";\n';
      pac_template += '  }\n';
    }
    pac_template += '  return "DIRECT";\n}';
    g_conf.pac_config_item_mode = pac_template;
    g_conf.pac_config = g_conf.pac_config_item_mode;
  } else if (g_conf.traffic_mode === "custom") {
    g_conf.pac_config = g_conf.pac_config_custom_mode;
  } else {
    g_conf.pac_config = g_conf_pac_config_full_mode;
  }
}

function _saveOptions() {
  if ($("#check_current_ip").prop("checked") == true) {
    g_conf.check_current_ip = true;
  } else {
    g_conf.check_current_ip = false;
  }
  if ($("#refresh_active_tab").prop("checked") == true) {
    g_conf.refresh_active_tab = true;
  } else {
    g_conf.refresh_active_tab = false;
  }
  if ($("#use_secure_connection").prop("checked") == true) {
    g_conf.use_secure_connection = true;
  } else {
    g_conf.use_secure_connection = false;
  }
  if ($("#use_ssl_bump").prop("checked") == true) {
    g_conf.use_ssl_bump = true;
  } else {
    g_conf.use_ssl_bump = false;
  }
  if ($("#remember_me").prop("checked") == true) {
    g_conf.remember_me = true;
  } else {
    g_conf.remember_me = false;
  }
  
  g_conf.username = $("#username").val();
  g_conf.password = $("#password").val();
  if ($("#mode_item").prop("checked")) {
    g_conf.traffic_mode = "item";
  } else if ($("#mode_custom").prop("checked")) {
    g_conf.traffic_mode = "custom";
  } else {
    g_conf.traffic_mode = "full";
  }
  generatePacConfig();
  sendPacConfigAndReconnect();
  sendAuth();
  chrome.storage.local.set(g_conf);
  $.notify(chrome.i18n.getMessage("i18n_248336101b461380a4b2391a7625493d"), {
    autoHideDelay: 1000,
    className: "success",
    position: "bottom left"
  });
}

var g_save_options_timer = null;

function saveOptions() {
  if (g_save_options_timer) {
    window.clearTimeout(g_save_options_timer);
  }
  g_save_options_timer = window.setTimeout(_saveOptions, 100);
}

var g_codemirror_editor = null;

function initCodeMirrorEditor() {
  if (g_codemirror_editor == null) {
    var pc = document.getElementById("pac_config");
    g_codemirror_editor = CodeMirror.fromTextArea(pc, {
      lineNumbers: true,
      matchBrackets: true
    });
    g_codemirror_editor.on('change', function (cm) {
      g_conf.pac_config_custom_mode = cm.getValue();
      saveOptions();
      sendPacConfig();
    });
  }
}

function changeMenu(menu) {
  $("#a_menu_mode").attr("class", "block py-3 px-4 text-gray-300 hover:bg-gray-700 hover:text-white rounded-sm flex items-center transition-colors");
  $("#a_menu_general").attr("class", "block py-3 px-4 text-gray-300 hover:bg-gray-700 hover:text-white rounded-sm flex items-center transition-colors");
  $("#a_menu_advanced").attr("class", "block py-3 px-4 text-gray-300 hover:bg-gray-700 hover:text-white rounded-sm flex items-center transition-colors");
  
  $("#mudfish_conf_options_menu_mode").addClass("hidden");
  $("#mudfish_conf_options_menu_general").addClass("hidden");
  $("#mudfish_conf_options_menu_advanced").addClass("hidden");
  
  if (menu == "mode") {
    $("#a_menu_mode").attr("class", "block py-3 px-4 bg-blue-500 text-white font-medium flex items-center transition-colors");
    $("#mudfish_conf_options_menu_mode").removeClass("hidden");
  } else if (menu == "general") {
    $("#a_menu_general").attr("class", "block py-3 px-4 bg-blue-500 text-white font-medium flex items-center transition-colors");
    $("#mudfish_conf_options_menu_general").removeClass("hidden");
  } else if (menu == "advanced") {
    $("#a_menu_advanced").attr("class", "block py-3 px-4 bg-blue-500 text-white font-medium flex items-center transition-colors");
    $("#mudfish_conf_options_menu_advanced").removeClass("hidden");
  }
}

function handleOptions() {
  restoreOptions();
  $("#a_menu_mode").click(function () {
    changeMenu("mode");
  });
  $("#a_menu_general").click(function () {
    changeMenu("general");
  });
  $("#a_menu_advanced").click(function () {
    changeMenu("advanced");
    initCodeMirrorEditor();
  });
  $("#remember_me").click(function () {
    if ($("#remember_me").prop("checked") == true) {
      $("#username").removeProp("disabled");
      $("#password").removeProp("disabled");
    } else {
      $("#username").prop("disabled", true);
      $("#password").prop("disabled", true);
      $("#username").val("");
      $("#password").val("");
    }
    saveOptions();
  });
  $("#check_current_ip").click(function () {
    saveOptions();
  });
  $("#refresh_active_tab").click(function () {
    saveOptions();
  });
  $("#pac_config_reset").click(function () {
    g_conf.pac_config_custom_mode = g_conf_pac_config_full_mode;
    if (g_codemirror_editor != null) {
      g_codemirror_editor.getDoc().setValue(g_conf.pac_config_custom_mode);
    }
  });
  $("#use_secure_connection").click(function () {
    $("#use_ssl_bump").removeProp("checked");
    saveOptions();
  });
  $("#use_ssl_bump").click(function () {
    $("#use_secure_connection").removeProp("checked");
    saveOptions();
  });
  $("#username").change(function () {
    saveOptions();
  });
  $("#password").change(function () {
    saveOptions();
  });
  $("#options_advanced_domain_dump_play").click(function () {
    if (g_proxy_domain_dump_enabled) {
      disableDumpDomain();
    } else {
      enableDumpDomain();
    }
  });
  $("#options_advanced_domain_dump_upload").click(function () {
    chrome.runtime.sendMessage({
      type: 'GET_DUMP_DOMAIN'
    }, function (response) {
      if (response.hostnames.length <= 0) {
        $.notify(chrome.i18n.getMessage("i18n_eb35bfb40ecd05f44c6d758a05cb78ee"), {
          className: "warn",
          position: "bottom left"
        });
        return;
      }
      var username = prompt(chrome.i18n.getMessage("i18n_4d379022242ec00e209c98f6f4876abb"));
      if (username === null || username.length <= 0) {
        return;
      }
      var reason = prompt(chrome.i18n.getMessage("i18n_f050fbfd3285f246e9dbbcc3004ef3d4"));
      if (reason === null || reason.length <= 0) {
        return;
      }
      $.ajax({
        url: "https://crs.mudfish.net/report/webextension_chrome_log.php",
        type: "POST",
        data: {
          reason: reason,
          username: username,
          hostnames: response.hostnames.join(",")
        },
        success: function (data) {
          chrome.runtime.sendMessage({
            type: 'CLEAR_DUMP_DOMAIN'
          }, function (response) {
            $.notify(chrome.i18n.getMessage("i18n_fe8d588f340d7507265417633ccff16e"), {
              className: "success",
              position: "bottom left"
            });
          });
        },
        error: function (xhr, op, err) {
          $.notify(chrome.i18n.getMessage("i18n_0561495323b38ac7e5df1e0f9fb37e90"), {
            className: "error",
            position: "bottom left"
          });
        }
      });
    });
  });
  
  $("input[name='traffic_mode']").change(function() {
    updateItemModeVisibility();
    saveOptions();
  });
  
  $("#goto_advanced_tab").click(function() {
    changeMenu("advanced");
    initCodeMirrorEditor();
  });
}

function options_i18n() {
  $("span[id^='i18n_']").each(function() {
    var id = $(this).attr("id");
    var message = chrome.i18n.getMessage(id);
    $(`[id="${id}"]`).html(message);
  });
}

function init_pac_app() {
  var pac_app_save = function() {
    var name = prompt(chrome.i18n.getMessage("i18n_df8a85d5c9fe070fd9f97d9dddc98366"));
    if (name === null || name.length == 0) {
      return;
    }
    if (g_codemirror_editor != null) {
      g_conf.pac_storage[name] = g_codemirror_editor.getDoc().getValue();
      chrome.storage.local.set(g_conf);
    }
  }
  var pac_app_remove = function(name) {
    if (confirm(chrome.i18n.getMessage("i18n_8e7759232170ff0b919833f59176d7a1"))) {
      delete g_conf.pac_storage[name];
      chrome.storage.local.set(g_conf);
    }
  }
  var pac_app_load = function(name) {
    if (g_codemirror_editor != null) {
      g_codemirror_editor.getDoc().setValue(g_conf.pac_storage[name]);
    }
  }

  m.mount(document.getElementById("pac_app"), {
    view: function() {
      return m("div", [
        m("div.font-bold.text-lg.mb-2", chrome.i18n.getMessage("i18n_d8660c0d91639c227b7b2eeb769b13c4")),
        m("p.text-sm.text-gray-600.mb-3", chrome.i18n.getMessage("i18n_8030a31cc05bb617ae29b23030d91158")),
        m("ul.list-none.pl-0", { style: { display: Object.keys(g_conf.pac_storage).length > 0 ? "none" : "" }},
          m("li.text-gray-500.italic", chrome.i18n.getMessage("i18n_74257abe77dbb81554bc99d7c592e133"))),
        m("ul.list-none.pl-0", { style: { display: Object.keys(g_conf.pac_storage).length > 0 ? "" : "none" }}, [
          Object.keys(g_conf.pac_storage).map(function(name) {
            return m("li.py-1.border-b.border-gray-200", [
              m("div.flex.justify-between.items-center", [
                m("a.text-blue-500.hover:text-blue-700.font-medium", { 
                  href: "#", 
                  onclick: function() { pac_app_load(name) }
                }, name),
                m("a.text-red-500.hover:text-red-700.ml-2.px-2", { 
                  href: "#", 
                  onclick: function() { pac_app_remove(name) }
                }, "×")
              ]),
            ]);
          })
        ]),
        m("button.mt-3.bg-yellow-500.hover:bg-yellow-600.text-white.py-1.px-3.rounded", { 
          onclick: pac_app_save 
        }, chrome.i18n.getMessage("i18n_c9cc8cce247e49bae79f15173ce97354"))
      ])
    }
  });
}

function updateItemModeVisibility() {
  if ($("#mode_item").prop("checked")) {
    $("#item_mode_config").removeClass("hidden");
    $("#custom_mode_hint").addClass("hidden");
    if (g_mudfish_items.length === 0) {
      mudfishItemFetch();
    }
  } else if ($("#mode_custom").prop("checked")) {
    $("#item_mode_config").addClass("hidden");
    $("#custom_mode_hint").removeClass("hidden");
  } else {
    $("#item_mode_config").addClass("hidden");
    $("#custom_mode_hint").addClass("hidden");
  }
}

$(document).ready(function () {
  options_i18n();
  handleOptions();
  changeMenu("mode");
});